chiark / gitweb /
[PATCH] fix stupid off-by-one bug that caused udevstart to die on x86-64 boxes.
[elogind.git] / namedev.c
index b3a2a8ae4b0bcb2bf0eb62955d47f71bf57153d8..52417e3793b655fcca041ff1ead58e7a0ba8d4ce 100644 (file)
--- a/namedev.c
+++ b/namedev.c
@@ -32,9 +32,7 @@
 #include <time.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
-#ifndef __KLIBC__
 #include <sys/sysinfo.h>
-#endif
 
 #include "libsysfs/sysfs/libsysfs.h"
 #include "list.h"
@@ -604,6 +602,42 @@ static int match_place(struct config_device *dev, struct sysfs_class_device *cla
        return 0;
 }
 
+static int whitelist_search(struct sysfs_class_device *class_dev)
+{
+       char *sysblock = "/sys/block";
+       int i;
+
+       static char *list[] = {
+               "nb",
+               "ram",
+               "loop",
+               "fd",
+               "md",
+               "dos_cd",
+               "double",
+               "flash",
+               "msd",
+               "rflash",
+               "rom",
+               "rrom",
+               "sbpcd",
+               "pcd",
+               "pf",
+               "scd",
+               "ubd",
+               NULL,
+       };
+
+       if (strncmp(class_dev->path, sysblock, strlen(sysblock)))
+               return 0;
+
+       for (i=0; list[i] != NULL; i++)
+               if (!strncmp(class_dev->name, list[i], strlen(list[i])))
+                       return 1;
+
+       return 0;
+}
+
 static struct sysfs_device *get_sysfs_device(struct sysfs_class_device *class_dev)
 {
        struct sysfs_device *sysfs_device;
@@ -617,8 +651,8 @@ static struct sysfs_device *get_sysfs_device(struct sysfs_class_device *class_de
         * all partitions have the symlink in its parent directory.
         * But we need to watch out for block devices that do not have parents, yet
         * look like a partition (fd0, loop0, etc.)  They all do not have a device
* symlink yet.  We do sit and spin on waiting for them right now, we should
* possibly have a whitelist for these devices here...
         */
        class_dev_parent = sysfs_get_classdev_parent(class_dev);
        if (class_dev_parent != NULL) 
@@ -628,8 +662,13 @@ static struct sysfs_device *get_sysfs_device(struct sysfs_class_device *class_de
        tspec.tv_nsec = 10000000;  /* sleep 10 millisec */
        loop = 10;
        while (loop--) {
-               if (udev_sleep)
+               if (udev_sleep) {
+                       if (whitelist_search(class_dev)) {
+                               sysfs_device = NULL;
+                               goto exit;
+                       }
                        nanosleep(&tspec, NULL);
+               }
 
                if (class_dev_parent)
                        sysfs_device = sysfs_get_classdev_device(class_dev_parent);
@@ -617,8 +651,8 @@ static struct sysfs_device *get_sysfs_device(struct sysfs_class_device *class_de
         * But we need to watch out for block devices that do not have parents, yet
         * look like a partition (fd0, loop0, etc.)  They all do not have a device
-        * symlink yet.  We do sit and spin on waiting for them right now, we should
-        * possibly have a whitelist for these devices here...
+        * symlink yet.  We do sit and spin on waiting for them right now unless
+        * they happen to be in the whitelist in which case we exit.
         */
        class_dev_parent = sysfs_get_classdev_parent(class_dev);